﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MatrixLibrary;
using Tangra.SDK;

namespace Tangra.Addins
{
	public class AstrometryAction : ITangraAddinAction
	{
		private ITangraApplication m_Application;

		public string DisplayName
		{
			get { return "Magnitude Tranformations Fitter"; }
		}

		public AddinActionType ActionType
		{
			get { return AddinActionType.Astrometry; }
		}

		public System.Drawing.Bitmap Icon
		{
			get { return null; }
		}

		public System.Drawing.Color IconTransparentColor
		{
			get { return System.Drawing.Color.Transparent; }
		}

		Dictionary<ulong, List<int>> m_MeasurementsPerStar = new Dictionary<ulong, List<int>>();
		private List<ITangraCatalogStar> m_CatalogStars = new List<ITangraCatalogStar>();

		public void Execute()
		{
			ITangraAstrometricSolution solution =  m_Application.GetCurrentFrameAstrometricSolution();
			List<ITangraMatchedStar> matchedStars = solution.GetAllMatchedStars();
			foreach(ITangraMatchedStar star in matchedStars)
			{
                if (star.CatalogStar != null && star.PSFAmplitude != -1)
				{
					List<int> currentMeasurements;
					if (!m_MeasurementsPerStar.TryGetValue(star.CatalogStar.StarNo, out currentMeasurements))
					{
						currentMeasurements = new List<int>();
						m_MeasurementsPerStar.Add(star.CatalogStar.StarNo, currentMeasurements);
						m_CatalogStars.Add(star.CatalogStar);
					}
					currentMeasurements.Add(star.PSFAmplitude);
				}
			}
		}

		public void Initialize(ITangraApplication application)
		{
			m_Application = application;
		}

		public void Finalize()
		{ }

		internal void OnBeginMultiFrameAstrometry()
		{
			m_MeasurementsPerStar.Clear();
			m_CatalogStars.Clear();
		}

		internal void OnEndMultiFrameAstrometry()
		{
			if (m_MeasurementsPerStar.Count > 0)
			{
				var frmMagnitudeFits = new frmMagnitudeFits(m_MeasurementsPerStar, m_CatalogStars);
				frmMagnitudeFits.StartPosition = FormStartPosition.CenterParent;
				frmMagnitudeFits.Show(m_Application.MainWindow);

				Form mainForm = Application.OpenForms
					.Cast<Form>()
					.FirstOrDefault(f => f.Handle == m_Application.MainWindow.Handle);

				if (mainForm != null)
				{
					frmMagnitudeFits.Top = mainForm.Top;
					frmMagnitudeFits.Left = mainForm.Right;
				}
			}
		}
	}
}
