﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Tangra.SDK;

namespace Tangra.Addins
{
	internal class LightCurveAddinAction : ITangraAddinAction
	{
		private ITangraApplication m_Application;
		private frmLightCurveAddin m_LightCurveAddinForm = null;
		private object m_SyncRoot = new object();

		public void Initialize(ITangraApplication application)
		{
			m_Application = application;
		}

		public void Finalize()
		{ }

		public void Execute()
		{
			ITangraLightCurve lightCurve = m_Application.GetOpenedLightCurve();
			if (lightCurve != null)
			{
				EnsureLightCurveFormThreadSafe(lightCurve);

                if (!m_LightCurveAddinForm.Visible)
                {
                    m_LightCurveAddinForm.StartPosition = FormStartPosition.CenterParent;
                    m_LightCurveAddinForm.Show(m_Application.LightCurvesWindow);
                }
			}
		}

		private void EnsureLightCurveFormThreadSafe(ITangraLightCurve lightCurve)
		{
			lock (m_SyncRoot)
			{
                if (m_LightCurveAddinForm == null || m_LightCurveAddinForm.IsDisposed)
					m_LightCurveAddinForm = new frmLightCurveAddin();

			    m_LightCurveAddinForm.Text = string.Format("Demonstration Addin (Tangra Version: {0})", m_Application.GetVersion().ToString());
				m_LightCurveAddinForm.SetLightCurve(lightCurve);
			}			
		}

		public string DisplayName
		{
			get { return "Demo Action"; }
		}

		public AddinActionType ActionType
		{
			get { return AddinActionType.LightCurve; }
		}

		public Bitmap Icon
		{
            get { { return Tangra.Addins.Properties.Resources.Icon1.ToBitmap(); } }
		}

		public Color IconTransparentColor
		{
			get { return Color.Transparent; }
		}

		internal void OnLightCurveDataRecomputed()
		{
			lock(m_SyncRoot)
			{
				if (m_LightCurveAddinForm != null)
					m_LightCurveAddinForm.OnLightCurveDataRecomputed();
			}
		}

        internal void OnLightCurveSelectedFrameChanged()
        {
            lock (m_SyncRoot)
            {
                if (m_LightCurveAddinForm != null)
                    m_LightCurveAddinForm.OnLightCurveSelectedFrameChanged();
            }            
        }
	}
}
