﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Tangra.SDK;

namespace Tangra.Addins
{
	public partial class frmLightCurveAddin : Form
	{
		private ITangraLightCurve m_LightCurve;
		private Bitmap m_Plot;
		 
		public frmLightCurveAddin()
		{
			InitializeComponent();

			m_Plot = new Bitmap(picPlot.Width, picPlot.Height);
		}

		internal void SetLightCurve(ITangraLightCurve lightCurve)
		{ 
			m_LightCurve = lightCurve;
		    nudFrameNo.Minimum = m_LightCurve.GenericInfo.FirstVideoFrame;
            nudFrameNo.Maximum = m_LightCurve.GenericInfo.LastVideoFrame;
		}

		private void frmLightCurveAddin_Load(object sender, EventArgs e)
		{
			PlotData();
            DisplaySelectedFrameInfo();
		}

		private void PlotData()
		{
			var objectData = new List<ITangraLightCurveDataPoint>[m_LightCurve.ObjectCount];
			for (int i = 0; i < m_LightCurve.ObjectCount; i++)
			{
				objectData[i] = m_LightCurve.GetDataPointsForObject(i);
			}

			float minMea = m_LightCurve.State.MinAdjustedReading;
            float maxMea = m_LightCurve.State.MaxAdjustedReading;

			int fivePercRange = (int)Math.Floor(0.05 * (maxMea - minMea));
			minMea -= fivePercRange;
			maxMea += fivePercRange;

            float coefX = picPlot.Width * 1.0f / (m_LightCurve.State.BinSize == 0
                    ? (float)(objectData[0][objectData[0].Count - 1].FrameNo - objectData[0][0].FrameNo)
                    : (float)(objectData[0][objectData[0].Count - 1].BinNo - objectData[0][0].BinNo));
			float coefY = picPlot.Height * 1.0f / (maxMea - minMea);

			var objBrushes = new Brush[] { Brushes.Blue, Brushes.Lime, Brushes.Orange, Brushes.Purple };

			using (Graphics g = Graphics.FromImage(m_Plot))
			{
				g.Clear(Color.WhiteSmoke);

				for (int i = 0; i < m_LightCurve.ObjectCount; i++)
				{
					foreach (ITangraLightCurveDataPoint dataPoint in objectData[i])
					{
						float x = coefX * (m_LightCurve.State.BinSize == 0
                            ? (float)(dataPoint.FrameNo - objectData[0][0].FrameNo)
                            : (float)(dataPoint.BinNo - objectData[0][0].BinNo));
						float y = picPlot.Height - coefY * (float)(dataPoint.Value - minMea);

						g.FillEllipse(objBrushes[i], x - 3, y - 3, 5, 5);
					}
				}				

				g.Save();
			}

			picPlot.Image = (Bitmap)m_Plot.Clone();
			picPlot.Update();
		}

        private void DisplaySelectedFrameInfo()
        {
            lblSelFrameNo.Text = m_LightCurve.State.SelectedFrameNo.ToString();
            if (m_LightCurve.State.SelectedFrameNo != -1)
                lblSelFrameTime.Text = m_LightCurve.GetTimeForFrame((uint)m_LightCurve.State.SelectedFrameNo).ToString("HH:mm:ss.fff");
        }

		internal void OnLightCurveDataRecomputed()
		{
			PlotData();
		}

        internal void OnLightCurveSelectedFrameChanged()
        {
            DisplaySelectedFrameInfo();
        }

        private void btnMoveToFrame_Click(object sender, EventArgs e)
        {
            m_LightCurve.SelectFrame((uint)nudFrameNo.Value);
        }
	}
}
