﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Tangra.Addins
{
    public partial class frmPixelDistribution : Form
    {
        private Dictionary<byte, float> m_Distribution = new Dictionary<byte, float>();
        private Dictionary<byte, float> m_DisplayData = new Dictionary<byte, float>();

        private List<Bitmap> m_AllFrames;

        internal frmPixelDistribution(List<Bitmap> allFrames, int firstFrame, int lastFrame)
        {
            InitializeComponent();

            m_AllFrames = allFrames;
            Text = string.Format("Histogram of All Pixels from Frames {0} - {1}", firstFrame, lastFrame);
        }

        private void PopulateDistributionFromBitmaps(List<Bitmap> allFrames)
        {
            m_Distribution.Clear();
            m_DisplayData.Clear();
            for (int i = 0; i <= 255; i++)
            {
                m_Distribution.Add((byte)i, 0);
                m_DisplayData.Add((byte)i, 0);
            }

            int colorIndex = 2; /* 0 = Blue, 1 = Green, 2 = Red, -1 = Gray Scale */

            int sum = 0;
            foreach (Bitmap bmp in allFrames)
            {
                BitmapData bmData = bmp.LockBits(new Rectangle(0, 0, bmp.Width, bmp.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
                try
                {
                    int stride = bmData.Stride;
                    System.IntPtr Scan0 = bmData.Scan0;

                    unsafe
                    {
                        byte* p = (byte*)(void*)Scan0;

                        int nOffset = stride - bmp.Width * 3;

                        for (int y = 0; y < bmp.Height; ++y)
                        {
                            for (int x = 0; x < bmp.Width; ++x)
                            {
                                byte bt;
                                if (colorIndex == -1)
                                    bt = (byte)(.299 * p[2] + .587 * p[1] + .114 * p[0]);
                                else
                                    bt = p[colorIndex];

                                m_Distribution[bt]++;
                                sum++;

                                p += 3;
                            }
                            p += nOffset;
                        }
                    }                    
                }
                finally
                {
                    bmp.UnlockBits(bmData);    
                }
            }

            lblPixCount.Text = string.Format("{0} million", (sum / 1000000.0).ToString("0.00"));              
        }
       
        private void PlotData()
        {
            for (int i = 0; i <= 255; i++)
            {
                if (rbScaleLog.Checked)
                    m_DisplayData[(byte)i] = (float)Math.Log10(m_Distribution[(byte)i] + 1);
                else
                    m_DisplayData[(byte)i] = m_Distribution[(byte)i];
            }    
        }

        private void frmPixelDistribution_Load(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            try
            {
                PopulateDistributionFromBitmaps(m_AllFrames);
                m_AllFrames.ForEach((b) => b.Dispose());

                PlotData();        
            }
            finally
            {
                Cursor = Cursors.Default;    
            }

            DrawHistogram();
        }

        private void DrawHistogram()
        {
            if (picHistogram.Image != null)
                picHistogram.Image.Dispose();

            picHistogram.Image = new Bitmap(picHistogram.Width, picHistogram.Height);

            float maxVal = m_DisplayData.Values.Max();
            int XGAP = 10;
            int YGAP = 10;

            using (Graphics g = Graphics.FromImage(picHistogram.Image))
            {
                float xScale = (picHistogram.Image.Width - 2 * XGAP) * 1.0f / 256;
                float yScale = (picHistogram.Image.Height - 2 * YGAP) * 1.0f / maxVal;

                g.FillRectangle(SystemBrushes.ControlDark,
                                new Rectangle(0, 0, picHistogram.Image.Width, picHistogram.Image.Height));
                g.DrawRectangle(Pens.Black, XGAP, YGAP, picHistogram.Image.Width - 2 * XGAP + 1, picHistogram.Image.Height - 2 * YGAP);

                foreach (byte key in m_DisplayData.Keys)
                {
                    float xFrom = XGAP + key * xScale + 1;
                    float xSize = xScale;
                    float yFrom = picHistogram.Image.Height - YGAP - m_DisplayData[key] * yScale;
                    float ySize = m_DisplayData[key] * yScale;

                    g.FillRectangle(Brushes.LimeGreen, xFrom, yFrom, xSize, ySize);
                }

                g.Save();
            }

            picHistogram.Refresh();
        }

        private void picHistogram_Resize(object sender, EventArgs e)
        {
            DrawHistogram();
        }

        private void rbScaleLinear_CheckedChanged(object sender, EventArgs e)
        {
            PlotData();
            DrawHistogram();
        }
    }
}
